"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .merchantaccountcreate import MerchantAccountCreate, MerchantAccountCreateTypedDict
from gr4vy.types import BaseModel
from gr4vy.utils import FieldMetadata, QueryParamMetadata, RequestMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateMerchantAccountRequestTypedDict(TypedDict):
    merchant_account_create: MerchantAccountCreateTypedDict
    timeout_in_seconds: NotRequired[float]


class CreateMerchantAccountRequest(BaseModel):
    merchant_account_create: Annotated[
        MerchantAccountCreate,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    timeout_in_seconds: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .shippingdetails import ShippingDetails, ShippingDetailsTypedDict
from gr4vy.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class CollectionNoCursorShippingDetailsTypedDict(TypedDict):
    items: List[ShippingDetailsTypedDict]
    r"""A list of items returned for this request."""


class CollectionNoCursorShippingDetails(BaseModel):
    items: List[ShippingDetails]
    r"""A list of items returned for this request."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


GiftCardErrorCode = Union[
    Literal[
        "invalid_gift_card",
        "expired_card",
        "inactive_card",
        "invalid_service_credentials",
        "invalid_amount",
        "incorrect_currency",
        "insufficient_funds",
        "invalid_service_configuration",
        "operation_canceled",
        "service_error",
        "service_network_error",
        "unknown_error",
    ],
    UnrecognizedStr,
]
r"""Gift card error codes.

Gr4vy normalised gift card error codes. Keep the naming and style in line with
those in the connectors framework.

If new codes are added, append them at the end or amend public simulator
documentation: https://docs.gr4vy.com/guides/features/gift-cards/simulator
"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class PayoutMerchantTypedDict(TypedDict):
    name: str
    r"""The name of the merchant."""
    identification_number: str
    r"""Unique value which identifies a merchant for processing transactions, also known as a MID."""
    phone_number: str
    r"""The phone number for the merchant which should be formatted according to the E164 number standard."""
    url: str
    r"""Merchant website URL."""
    statement_descriptor: str
    r"""Value to explain charges or payments on bank statements."""
    merchant_category_code: str
    r"""Merchant classification for the type of goods or services it provides."""
    address: NotRequired[Nullable[AddressTypedDict]]
    r"""The address for the merchant."""


class PayoutMerchant(BaseModel):
    name: str
    r"""The name of the merchant."""

    identification_number: str
    r"""Unique value which identifies a merchant for processing transactions, also known as a MID."""

    phone_number: str
    r"""The phone number for the merchant which should be formatted according to the E164 number standard."""

    url: str
    r"""Merchant website URL."""

    statement_descriptor: str
    r"""Value to explain charges or payments on bank statements."""

    merchant_category_code: str
    r"""Merchant classification for the type of goods or services it provides."""

    address: OptionalNullable[Address] = UNSET
    r"""The address for the merchant."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["address"]
        nullable_fields = ["address"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



import hmac
import hashlib
import time
from typing import Optional

def verify_webhook(
    payload: str,
    secret: str,
    signature_header: Optional[str],
    timestamp_header: Optional[str],
    timestamp_tolerance: int
) -> None:
    """
    Verifies a webhook signature.

    :param payload: The payload that was sent in the webhook.
    :param secret: The secret that was used to sign the webhook.
    :param signature_header: The header that contains the signature.
    :param timestamp_header: The header that contains the timestamp.
    :param timestamp_tolerance: The tolerance for the timestamp in seconds.
    :raises ValueError: If the signature or timestamp is invalid.
    """
    if not signature_header or not timestamp_header:
        raise ValueError("Missing header values")

    try:
        timestamp = int(timestamp_header)
    except ValueError as exc:
        raise ValueError("Invalid header timestamp") from exc

    signatures = signature_header.split(",")
    expected_signature = hmac.new(
        secret.encode(),
        f"{timestamp}.{payload}".encode(),
        hashlib.sha256
    ).hexdigest()

    if expected_signature not in signatures:
        raise ValueError("No matching signature found")

    if timestamp_tolerance and timestamp < time.time() - timestamp_tolerance:
        raise ValueError("Timestamp too old")



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Any, Literal
from typing_extensions import Annotated, NotRequired, TypedDict


class ClickToPayFPANPaymentMethodCreateTypedDict(TypedDict):
    r"""Create a Click to Pay payment with an FPAN or virtual PAN"""

    expiration_date: str
    r"""The expiration date of the card, formatted `MM/YY`."""
    number: str
    r"""The 13-19 digit number for this card."""
    buyer_external_identifier: NotRequired[Nullable[str]]
    r"""The external identifier of the buyer to attach the method to."""
    buyer_id: NotRequired[Nullable[str]]
    r"""The ID of the buyer to attach the method to."""
    external_identifier: NotRequired[Nullable[str]]
    r"""The merchant reference for this payment method."""
    card_type: NotRequired[Nullable[str]]
    r"""The type of the card used"""
    method: Literal["click-to-pay"]
    r"""Aways `click-to-pay`."""
    redirect_url: NotRequired[Nullable[str]]
    r"""The URL to redirect a user back to after the complete 3DS in browser."""
    security_code: NotRequired[Nullable[Any]]
    r"""The 3 or 4 digit security code often found on the card. This often referred to as the CVV or CVD."""


class ClickToPayFPANPaymentMethodCreate(BaseModel):
    r"""Create a Click to Pay payment with an FPAN or virtual PAN"""

    expiration_date: str
    r"""The expiration date of the card, formatted `MM/YY`."""

    number: str
    r"""The 13-19 digit number for this card."""

    buyer_external_identifier: OptionalNullable[str] = UNSET
    r"""The external identifier of the buyer to attach the method to."""

    buyer_id: OptionalNullable[str] = UNSET
    r"""The ID of the buyer to attach the method to."""

    external_identifier: OptionalNullable[str] = UNSET
    r"""The merchant reference for this payment method."""

    card_type: OptionalNullable[str] = UNSET
    r"""The type of the card used"""

    METHOD: Annotated[
        Annotated[
            Literal["click-to-pay"], AfterValidator(validate_const("click-to-pay"))
        ],
        pydantic.Field(alias="method"),
    ] = "click-to-pay"
    r"""Aways `click-to-pay`."""

    redirect_url: OptionalNullable[str] = UNSET
    r"""The URL to redirect a user back to after the complete 3DS in browser."""

    security_code: OptionalNullable[Any] = UNSET
    r"""The 3 or 4 digit security code often found on the card. This often referred to as the CVV or CVD."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "buyer_external_identifier",
            "buyer_id",
            "external_identifier",
            "card_type",
            "redirect_url",
            "security_code",
        ]
        nullable_fields = [
            "buyer_external_identifier",
            "buyer_id",
            "external_identifier",
            "card_type",
            "redirect_url",
            "security_code",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .paymentservicetoken import PaymentServiceToken, PaymentServiceTokenTypedDict
from gr4vy.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class CollectionNoCursorPaymentServiceTokenTypedDict(TypedDict):
    items: List[PaymentServiceTokenTypedDict]
    r"""A list of items returned for this request."""


class CollectionNoCursorPaymentServiceToken(BaseModel):
    items: List[PaymentServiceToken]
    r"""A list of items returned for this request."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CheckoutSessionPaymentMethodCreateTypedDict(TypedDict):
    id: str
    r"""The ID for the checkout session."""
    method: Literal["checkout-session"]
    r"""Always `checkout-session`"""
    external_identifier: NotRequired[Nullable[str]]
    r"""The merchant reference that can be used to match the payment method against your own records."""
    buyer_id: NotRequired[Nullable[str]]
    r"""The `id` of a stored buyer to use Use this instead of the `buyer_external_identifier`."""
    buyer_external_identifier: NotRequired[Nullable[str]]
    r"""The `external_identifier` of a stored buyer to use. Use this instead of the `buyer_id`."""


class CheckoutSessionPaymentMethodCreate(BaseModel):
    id: str
    r"""The ID for the checkout session."""

    METHOD: Annotated[
        Annotated[
            Optional[Literal["checkout-session"]],
            AfterValidator(validate_const("checkout-session")),
        ],
        pydantic.Field(alias="method"),
    ] = "checkout-session"
    r"""Always `checkout-session`"""

    external_identifier: OptionalNullable[str] = UNSET
    r"""The merchant reference that can be used to match the payment method against your own records."""

    buyer_id: OptionalNullable[str] = UNSET
    r"""The `id` of a stored buyer to use Use this instead of the `buyer_external_identifier`."""

    buyer_external_identifier: OptionalNullable[str] = UNSET
    r"""The `external_identifier` of a stored buyer to use. Use this instead of the `buyer_id`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "method",
            "external_identifier",
            "buyer_id",
            "buyer_external_identifier",
        ]
        nullable_fields = [
            "external_identifier",
            "buyer_id",
            "buyer_external_identifier",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .apierror import APIError
from .error400 import Error400, Error400Data
from .error401 import Error401, Error401Data
from .error403 import Error403, Error403Data
from .error404 import Error404, Error404Data
from .error405 import Error405, Error405Data
from .error409 import Error409, Error409Data
from .error425 import Error425, Error425Data
from .error429 import Error429, Error429Data
from .error500 import Error500, Error500Data
from .error502 import Error502, Error502Data
from .error504 import Error504, Error504Data
from .httpvalidationerror import HTTPValidationError, HTTPValidationErrorData


__all__ = [
    "APIError",
    "Error400",
    "Error400Data",
    "Error401",
    "Error401Data",
    "Error403",
    "Error403Data",
    "Error404",
    "Error404Data",
    "Error405",
    "Error405Data",
    "Error409",
    "Error409Data",
    "Error425",
    "Error425Data",
    "Error429",
    "Error429Data",
    "Error500",
    "Error500Data",
    "Error502",
    "Error502Data",
    "Error504",
    "Error504Data",
    "HTTPValidationError",
    "HTTPValidationErrorData",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from gr4vy import errors, models, utils
from gr4vy._hooks import HookContext
from gr4vy.types import OptionalNullable, UNSET
from gr4vy.utils import get_security_from_env
from typing import Any, Mapping, Optional


class Domains(BaseSDK):
    def create(
        self,
        *,
        digital_wallet_id: str,
        domain_name: str,
        timeout_in_seconds: Optional[float] = 1,
        merchant_account_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Any:
        r"""Register a digital wallet domain

        Register a digital wallet domain (Apple Pay only).

        :param digital_wallet_id: The ID of the digital wallet to remove a domain for.
        :param domain_name: The domain to add or remove.
        :param timeout_in_seconds:
        :param merchant_account_id: The ID of the merchant account to use for this request.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RegisterDigitalWalletDomainRequest(
            digital_wallet_id=digital_wallet_id,
            timeout_in_seconds=timeout_in_seconds,
            merchant_account_id=merchant_account_id,
            digital_wallet_domain=models.DigitalWalletDomain(
                domain_name=domain_name,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/digital-wallets/{digital_wallet_id}/domains",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.RegisterDigitalWalletDomainGlobals(
                merchant_account_id=self.sdk_configuration.globals.merchant_account_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.digital_wallet_domain,
                False,
                False,
                "json",
                models.DigitalWalletDomain,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="register_digital_wallet_domain",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Any)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error400Data)
            raise errors.Error400(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error401Data)
            raise errors.Error401(data=response_data)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error403Data)
            raise errors.Error403(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error404Data)
            raise errors.Error404(data=response_data)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error405Data)
            raise errors.Error405(data=response_data)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error409Data)
            raise errors.Error409(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.HTTPValidationErrorData
            )
            raise errors.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error425Data)
            raise errors.Error425(data=response_data)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error429Data)
            raise errors.Error429(data=response_data)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error500Data)
            raise errors.Error500(data=response_data)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error502Data)
            raise errors.Error502(data=response_data)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error504Data)
            raise errors.Error504(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def create_async(
        self,
        *,
        digital_wallet_id: str,
        domain_name: str,
        timeout_in_seconds: Optional[float] = 1,
        merchant_account_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Any:
        r"""Register a digital wallet domain

        Register a digital wallet domain (Apple Pay only).

        :param digital_wallet_id: The ID of the digital wallet to remove a domain for.
        :param domain_name: The domain to add or remove.
        :param timeout_in_seconds:
        :param merchant_account_id: The ID of the merchant account to use for this request.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RegisterDigitalWalletDomainRequest(
            digital_wallet_id=digital_wallet_id,
            timeout_in_seconds=timeout_in_seconds,
            merchant_account_id=merchant_account_id,
            digital_wallet_domain=models.DigitalWalletDomain(
                domain_name=domain_name,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/digital-wallets/{digital_wallet_id}/domains",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.RegisterDigitalWalletDomainGlobals(
                merchant_account_id=self.sdk_configuration.globals.merchant_account_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.digital_wallet_domain,
                False,
                False,
                "json",
                models.DigitalWalletDomain,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="register_digital_wallet_domain",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Any)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error400Data)
            raise errors.Error400(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error401Data)
            raise errors.Error401(data=response_data)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error403Data)
            raise errors.Error403(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error404Data)
            raise errors.Error404(data=response_data)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error405Data)
            raise errors.Error405(data=response_data)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error409Data)
            raise errors.Error409(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.HTTPValidationErrorData
            )
            raise errors.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error425Data)
            raise errors.Error425(data=response_data)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error429Data)
            raise errors.Error429(data=response_data)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error500Data)
            raise errors.Error500(data=response_data)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error502Data)
            raise errors.Error502(data=response_data)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error504Data)
            raise errors.Error504(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def delete(
        self,
        *,
        digital_wallet_id: str,
        domain_name: str,
        timeout_in_seconds: Optional[float] = 1,
        merchant_account_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Any:
        r"""Remove a digital wallet domain

        Remove a digital wallet domain (Apple Pay only).

        :param digital_wallet_id:
        :param domain_name: The domain to add or remove.
        :param timeout_in_seconds:
        :param merchant_account_id: The ID of the merchant account to use for this request.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UnregisterDigitalWalletDomainRequest(
            digital_wallet_id=digital_wallet_id,
            timeout_in_seconds=timeout_in_seconds,
            merchant_account_id=merchant_account_id,
            digital_wallet_domain=models.DigitalWalletDomain(
                domain_name=domain_name,
            ),
        )

        req = self._build_request(
            method="DELETE",
            path="/digital-wallets/{digital_wallet_id}/domains",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.UnregisterDigitalWalletDomainGlobals(
                merchant_account_id=self.sdk_configuration.globals.merchant_account_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.digital_wallet_domain,
                False,
                False,
                "json",
                models.DigitalWalletDomain,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="unregister_digital_wallet_domain",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Any)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error400Data)
            raise errors.Error400(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error401Data)
            raise errors.Error401(data=response_data)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error403Data)
            raise errors.Error403(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error404Data)
            raise errors.Error404(data=response_data)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error405Data)
            raise errors.Error405(data=response_data)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error409Data)
            raise errors.Error409(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.HTTPValidationErrorData
            )
            raise errors.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error425Data)
            raise errors.Error425(data=response_data)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error429Data)
            raise errors.Error429(data=response_data)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error500Data)
            raise errors.Error500(data=response_data)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error502Data)
            raise errors.Error502(data=response_data)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error504Data)
            raise errors.Error504(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def delete_async(
        self,
        *,
        digital_wallet_id: str,
        domain_name: str,
        timeout_in_seconds: Optional[float] = 1,
        merchant_account_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Any:
        r"""Remove a digital wallet domain

        Remove a digital wallet domain (Apple Pay only).

        :param digital_wallet_id:
        :param domain_name: The domain to add or remove.
        :param timeout_in_seconds:
        :param merchant_account_id: The ID of the merchant account to use for this request.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UnregisterDigitalWalletDomainRequest(
            digital_wallet_id=digital_wallet_id,
            timeout_in_seconds=timeout_in_seconds,
            merchant_account_id=merchant_account_id,
            digital_wallet_domain=models.DigitalWalletDomain(
                domain_name=domain_name,
            ),
        )

        req = self._build_request_async(
            method="DELETE",
            path="/digital-wallets/{digital_wallet_id}/domains",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.UnregisterDigitalWalletDomainGlobals(
                merchant_account_id=self.sdk_configuration.globals.merchant_account_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.digital_wallet_domain,
                False,
                False,
                "json",
                models.DigitalWalletDomain,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="unregister_digital_wallet_domain",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, Any)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error400Data)
            raise errors.Error400(data=response_data)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error401Data)
            raise errors.Error401(data=response_data)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error403Data)
            raise errors.Error403(data=response_data)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error404Data)
            raise errors.Error404(data=response_data)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error405Data)
            raise errors.Error405(data=response_data)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error409Data)
            raise errors.Error409(data=response_data)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, errors.HTTPValidationErrorData
            )
            raise errors.HTTPValidationError(data=response_data)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error425Data)
            raise errors.Error425(data=response_data)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error429Data)
            raise errors.Error429(data=response_data)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error500Data)
            raise errors.Error500(data=response_data)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error502Data)
            raise errors.Error502(data=response_data)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = utils.unmarshal_json(http_res.text, errors.Error504Data)
            raise errors.Error504(data=response_data)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .collection_transactionsummary_ import (
    CollectionTransactionSummary,
    CollectionTransactionSummaryTypedDict,
)
from .method import Method
from .transactionpaymentsource import TransactionPaymentSource
from .transactionstatus import TransactionStatus
from datetime import datetime
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import (
    FieldMetadata,
    HeaderMetadata,
    QueryParamMetadata,
    validate_open_enum,
)
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Callable, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ListTransactionsGlobalsTypedDict(TypedDict):
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class ListTransactionsGlobals(BaseModel):
    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""


class ListTransactionsRequestTypedDict(TypedDict):
    cursor: NotRequired[Nullable[str]]
    r"""A pointer to the page of results to return."""
    limit: NotRequired[int]
    r"""The maximum number of items that are at returned."""
    created_at_lte: NotRequired[Nullable[datetime]]
    r"""Filters the results to only transactions created before this ISO date-time string. The time zone must be included. Ensure that the date-time string is URL encoded, e.g. `2022-01-01T12:00:00+08:00` must be encoded as `2022-01-01T12%3A00%3A00%2B08%3A00`."""
    created_at_gte: NotRequired[Nullable[datetime]]
    r"""Filters the results to only transactions created after this ISO date-time string. The time zone must be included. Ensure that the date-time string is URL encoded, e.g. `2022-01-01T12:00:00+08:00` must be encoded as `2022-01-01T12%3A00%3A00%2B08%3A00`."""
    updated_at_lte: NotRequired[Nullable[datetime]]
    r"""Filters the results to only transactions updated before this ISO date-time string. The time zone must be included. Ensure that the date-time string is URL encoded, e.g. `2022-01-01T12:00:00+08:00` must be encoded as `2022-01-01T12%3A00%3A00%2B08%3A00`."""
    updated_at_gte: NotRequired[Nullable[datetime]]
    r"""Filters the results to only transactions updated after this ISO date-time string. The time zone must be included. Ensure that the date-time string is URL encoded, e.g. `2022-01-01T12:00:00+08:00` must be encoded as `2022-01-01T12%3A00%3A00%2B08%3A00`."""
    search: NotRequired[Nullable[str]]
    buyer_external_identifier: NotRequired[Nullable[str]]
    buyer_id: NotRequired[Nullable[str]]
    buyer_email_address: NotRequired[Nullable[str]]
    status: NotRequired[Nullable[List[TransactionStatus]]]
    r"""Filters the results to only the transactions that have a `status` that matches with any of the provided status values."""
    id: NotRequired[Nullable[str]]
    payment_service_transaction_id: NotRequired[Nullable[str]]
    external_identifier: NotRequired[Nullable[str]]
    metadata: NotRequired[Nullable[List[str]]]
    r"""Filters for transactions where their `metadata` values contain all of the provided `metadata` keys. The value sent for `metadata` must be formatted as a JSON string, and all keys and values must be strings. This value should also be URL encoded."""
    amount_eq: NotRequired[Nullable[int]]
    r"""Filters for transactions that have an `amount` that is equal to the provided `amount_eq` value."""
    amount_lte: NotRequired[Nullable[int]]
    r"""Filters for transactions that have an `amount` that is less than or equal to the `amount_lte` value."""
    amount_gte: NotRequired[Nullable[int]]
    r"""Filters for transactions that have an `amount` that is greater than or equal to the `amount_gte` value."""
    currency: NotRequired[Nullable[List[str]]]
    r"""Filters for transactions that have matching `currency` values. The `currency` values provided must be formatted as 3-letter ISO currency code."""
    payment_service_id: NotRequired[Nullable[List[str]]]
    r"""Filters for transactions that were processed by the provided `payment_service_id` values."""
    payment_method_id: NotRequired[Nullable[str]]
    payment_method_label: NotRequired[Nullable[str]]
    payment_method_fingerprint: NotRequired[Nullable[str]]
    method: NotRequired[Nullable[List[Method]]]
    r"""Filters for transactions that have matching `method` values."""
    error_code: NotRequired[Nullable[List[str]]]
    r"""Filters for transactions where the `error_code` matches one for the provided values."""
    has_refunds: NotRequired[Nullable[bool]]
    r"""Filters for transactions with refunds."""
    pending_review: NotRequired[Nullable[bool]]
    r"""Filters for transactions with a pending manual anti-fraud review."""
    checkout_session_id: NotRequired[Nullable[str]]
    r"""Filters for transactions where the `checkout_session_id` matches the provided value."""
    reconciliation_id: NotRequired[Nullable[str]]
    r"""Filters for transactions where the `reconciliation_id` matches the provided value."""
    has_gift_card_redemptions: NotRequired[Nullable[bool]]
    r"""Filters for transactions with gift card redemptions."""
    gift_card_id: NotRequired[Nullable[str]]
    r"""Filters for transactions where a gift card used has an `id` that matches the provided value."""
    gift_card_last4: NotRequired[Nullable[str]]
    r"""Filters for transactions that have at least one gift card redemption where the last 4 digits of its gift card number matches exactly with the provided value."""
    has_settlements: NotRequired[Nullable[bool]]
    r"""Filters for transactions that have at least one associated settlement record."""
    payment_method_bin: NotRequired[Nullable[str]]
    r"""Filter for transactions that have a card with a BIN that matches exactly with the provided value."""
    payment_source: NotRequired[Nullable[List[TransactionPaymentSource]]]
    r"""Filters the results to only the transactions that have a payment source that matches with any of the provided values."""
    is_subsequent_payment: NotRequired[Nullable[bool]]
    r"""Filters for transactions where the `is_subsequent_payment` matches the provided value."""
    merchant_initiated: NotRequired[Nullable[bool]]
    r"""Filters for transactions where the `merchant_initiated` matches the provided value."""
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class ListTransactionsRequest(BaseModel):
    cursor: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""A pointer to the page of results to return."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 20
    r"""The maximum number of items that are at returned."""

    created_at_lte: Annotated[
        OptionalNullable[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters the results to only transactions created before this ISO date-time string. The time zone must be included. Ensure that the date-time string is URL encoded, e.g. `2022-01-01T12:00:00+08:00` must be encoded as `2022-01-01T12%3A00%3A00%2B08%3A00`."""

    created_at_gte: Annotated[
        OptionalNullable[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters the results to only transactions created after this ISO date-time string. The time zone must be included. Ensure that the date-time string is URL encoded, e.g. `2022-01-01T12:00:00+08:00` must be encoded as `2022-01-01T12%3A00%3A00%2B08%3A00`."""

    updated_at_lte: Annotated[
        OptionalNullable[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters the results to only transactions updated before this ISO date-time string. The time zone must be included. Ensure that the date-time string is URL encoded, e.g. `2022-01-01T12:00:00+08:00` must be encoded as `2022-01-01T12%3A00%3A00%2B08%3A00`."""

    updated_at_gte: Annotated[
        OptionalNullable[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters the results to only transactions updated after this ISO date-time string. The time zone must be included. Ensure that the date-time string is URL encoded, e.g. `2022-01-01T12:00:00+08:00` must be encoded as `2022-01-01T12%3A00%3A00%2B08%3A00`."""

    search: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    buyer_external_identifier: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    buyer_id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    buyer_email_address: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    status: Annotated[
        OptionalNullable[
            List[
                Annotated[TransactionStatus, PlainValidator(validate_open_enum(False))]
            ]
        ],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters the results to only the transactions that have a `status` that matches with any of the provided status values."""

    id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    payment_service_transaction_id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    external_identifier: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    metadata: Annotated[
        OptionalNullable[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions where their `metadata` values contain all of the provided `metadata` keys. The value sent for `metadata` must be formatted as a JSON string, and all keys and values must be strings. This value should also be URL encoded."""

    amount_eq: Annotated[
        OptionalNullable[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that have an `amount` that is equal to the provided `amount_eq` value."""

    amount_lte: Annotated[
        OptionalNullable[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that have an `amount` that is less than or equal to the `amount_lte` value."""

    amount_gte: Annotated[
        OptionalNullable[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that have an `amount` that is greater than or equal to the `amount_gte` value."""

    currency: Annotated[
        OptionalNullable[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that have matching `currency` values. The `currency` values provided must be formatted as 3-letter ISO currency code."""

    payment_service_id: Annotated[
        OptionalNullable[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that were processed by the provided `payment_service_id` values."""

    payment_method_id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    payment_method_label: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    payment_method_fingerprint: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    method: Annotated[
        OptionalNullable[
            List[Annotated[Method, PlainValidator(validate_open_enum(False))]]
        ],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that have matching `method` values."""

    error_code: Annotated[
        OptionalNullable[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions where the `error_code` matches one for the provided values."""

    has_refunds: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions with refunds."""

    pending_review: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions with a pending manual anti-fraud review."""

    checkout_session_id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions where the `checkout_session_id` matches the provided value."""

    reconciliation_id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions where the `reconciliation_id` matches the provided value."""

    has_gift_card_redemptions: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions with gift card redemptions."""

    gift_card_id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions where a gift card used has an `id` that matches the provided value."""

    gift_card_last4: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that have at least one gift card redemption where the last 4 digits of its gift card number matches exactly with the provided value."""

    has_settlements: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that have at least one associated settlement record."""

    payment_method_bin: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter for transactions that have a card with a BIN that matches exactly with the provided value."""

    payment_source: Annotated[
        OptionalNullable[
            List[
                Annotated[
                    TransactionPaymentSource, PlainValidator(validate_open_enum(False))
                ]
            ]
        ],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters the results to only the transactions that have a payment source that matches with any of the provided values."""

    is_subsequent_payment: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions where the `is_subsequent_payment` matches the provided value."""

    merchant_initiated: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions where the `merchant_initiated` matches the provided value."""

    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "cursor",
            "limit",
            "created_at_lte",
            "created_at_gte",
            "updated_at_lte",
            "updated_at_gte",
            "search",
            "buyer_external_identifier",
            "buyer_id",
            "buyer_email_address",
            "status",
            "id",
            "payment_service_transaction_id",
            "external_identifier",
            "metadata",
            "amount_eq",
            "amount_lte",
            "amount_gte",
            "currency",
            "payment_service_id",
            "payment_method_id",
            "payment_method_label",
            "payment_method_fingerprint",
            "method",
            "error_code",
            "has_refunds",
            "pending_review",
            "checkout_session_id",
            "reconciliation_id",
            "has_gift_card_redemptions",
            "gift_card_id",
            "gift_card_last4",
            "has_settlements",
            "payment_method_bin",
            "payment_source",
            "is_subsequent_payment",
            "merchant_initiated",
            "merchant_account_id",
        ]
        nullable_fields = [
            "cursor",
            "created_at_lte",
            "created_at_gte",
            "updated_at_lte",
            "updated_at_gte",
            "search",
            "buyer_external_identifier",
            "buyer_id",
            "buyer_email_address",
            "status",
            "id",
            "payment_service_transaction_id",
            "external_identifier",
            "metadata",
            "amount_eq",
            "amount_lte",
            "amount_gte",
            "currency",
            "payment_service_id",
            "payment_method_id",
            "payment_method_label",
            "payment_method_fingerprint",
            "method",
            "error_code",
            "has_refunds",
            "pending_review",
            "checkout_session_id",
            "reconciliation_id",
            "has_gift_card_redemptions",
            "gift_card_id",
            "gift_card_last4",
            "has_settlements",
            "payment_method_bin",
            "payment_source",
            "is_subsequent_payment",
            "merchant_initiated",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ListTransactionsResponseTypedDict(TypedDict):
    result: CollectionTransactionSummaryTypedDict


class ListTransactionsResponse(BaseModel):
    next: Callable[[], Optional[ListTransactionsResponse]]

    result: CollectionTransactionSummary



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


Method = Union[
    Literal[
        "abitab",
        "affirm",
        "afterpay",
        "alipay",
        "alipayhk",
        "applepay",
        "arcuspaynetwork",
        "bacs",
        "bancontact",
        "banked",
        "bcp",
        "becs",
        "bitpay",
        "boleto",
        "boost",
        "card",
        "cashapp",
        "chaseorbital",
        "clearpay",
        "click-to-pay",
        "dana",
        "dcb",
        "dlocal",
        "ebanx",
        "efecty",
        "eps",
        "everydaypay",
        "gcash",
        "gem",
        "gemds",
        "gift-card",
        "giropay",
        "givingblock",
        "gocardless",
        "googlepay",
        "googlepay_pan_only",
        "gopay",
        "grabpay",
        "ideal",
        "kakaopay",
        "kcp",
        "khipu",
        "klarna",
        "latitude",
        "latitudeds",
        "laybuy",
        "linepay",
        "linkaja",
        "maybankqrpay",
        "mercadopago",
        "multibanco",
        "multipago",
        "nequi",
        "netbanking",
        "network-token",
        "nupay",
        "oney_10x",
        "oney_12x",
        "oney_3x",
        "oney_4x",
        "oney_6x",
        "ovo",
        "oxxo",
        "pagoefectivo",
        "payid",
        "paymaya",
        "paypal",
        "paypalpaylater",
        "payto",
        "payvalida",
        "picpay",
        "pix",
        "pse",
        "rabbitlinepay",
        "razorpay",
        "rapipago",
        "redpagos",
        "scalapay",
        "sepa",
        "servipag",
        "shopeepay",
        "singteldash",
        "smartpay",
        "sofort",
        "spei",
        "stitch",
        "stripedd",
        "thaiqr",
        "touchngo",
        "truemoney",
        "trustly",
        "trustlyeurope",
        "upi",
        "venmo",
        "vipps",
        "waave",
        "webpay",
        "wechat",
        "yape",
        "zippay",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .paymentoptioncontextapprovalui import (
    PaymentOptionContextApprovalUI,
    PaymentOptionContextApprovalUITypedDict,
)
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


RequiredFields1TypedDict = TypeAliasType("RequiredFields1TypedDict", Union[bool, Any])


RequiredFields1 = TypeAliasType("RequiredFields1", Union[bool, Any])


RequiredFields2TypedDict = TypeAliasType(
    "RequiredFields2TypedDict", Union[bool, Dict[str, RequiredFields1TypedDict]]
)


RequiredFields2 = TypeAliasType(
    "RequiredFields2", Union[bool, Dict[str, RequiredFields1]]
)


class PaymentOptionContextTypedDict(TypedDict):
    redirect_requires_popup: bool
    requires_tokenized_redirect_popup: bool
    approval_ui: NotRequired[Nullable[PaymentOptionContextApprovalUITypedDict]]
    required_fields: NotRequired[Nullable[Dict[str, RequiredFields2TypedDict]]]


class PaymentOptionContext(BaseModel):
    redirect_requires_popup: bool

    requires_tokenized_redirect_popup: bool

    approval_ui: OptionalNullable[PaymentOptionContextApprovalUI] = UNSET

    required_fields: OptionalNullable[Dict[str, RequiredFields2]] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["approval_ui", "required_fields"]
        nullable_fields = ["approval_ui", "required_fields"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


TransactionPaymentSource = Union[
    Literal["ecommerce", "moto", "recurring", "installment", "card_on_file"],
    UnrecognizedStr,
]
r"""The way payment method information made it to this transaction."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .merchantaccountupdate import MerchantAccountUpdate, MerchantAccountUpdateTypedDict
from gr4vy.types import BaseModel
from gr4vy.utils import (
    FieldMetadata,
    PathParamMetadata,
    QueryParamMetadata,
    RequestMetadata,
)
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UpdateMerchantAccountRequestTypedDict(TypedDict):
    merchant_account_id: str
    r"""The ID of the merchant account"""
    merchant_account_update: MerchantAccountUpdateTypedDict
    timeout_in_seconds: NotRequired[float]


class UpdateMerchantAccountRequest(BaseModel):
    merchant_account_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The ID of the merchant account"""

    merchant_account_update: Annotated[
        MerchantAccountUpdate,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    timeout_in_seconds: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
